﻿
Imports System.IO.Ports
Imports System.Text

Module Module_COM

    Friend ShowRawChars As Boolean = False
    Friend CaliperDivision As Single = 100

    Private WithEvents ComPort As New SerialPort()

    Friend Sub COM_Open(ByVal PortName As String, ByVal BaudRate As Int32)
        If PortName = "" Then PortName = "COM1"
        If BaudRate < 600 Then BaudRate = 600
        ' -------------------------------------------- If the port is Open then close it
        COM_Close()
        ComPort.PortName = PortName
        ComPort.BaudRate = BaudRate
        ComPort.Parity = Parity.None
        ComPort.DataBits = 8
        ComPort.StopBits = StopBits.One
        ' -------------------------------------------- Set params
        ComPort.ReceivedBytesThreshold = 1
        ComPort.WriteTimeout = 500
        ComPort.ReadTimeout = 500
        ' -------------------------------------------- minimum serial buffer is about 65536
        ' ComPort.ReadBufferSize = 4096
        ' ReadBufferSize default is 4096
        ' ReadBufferSize ignores values less than 4096
        ' The ReadBufferSize is the Windows buffer only, 
        ' The BytesToRead property is: SerialPort buffer + Windows buffer (65536 total)
        '
        ' -------------------------------------------- separator used by "ReadLine" (default is LF)
        'ComPort.NewLine = vbCrLf
        ' -------------------------------------------- 
        If FieldSeparator = "0" Or FieldSeparator = "1" Then
            ComPort.RtsEnable = True
            ComPort.DtrEnable = False
        ElseIf FieldSeparator = "2" Then
            ComPort.DtrEnable = True
            ComPort.RtsEnable = False
        Else
            ComPort.DtrEnable = False
            ComPort.RtsEnable = False
        End If
        ' --------------------------------------------
        Try
            ComPort.Open()
        Catch
        End Try
    End Sub

    Friend Sub COM_Close()
        If ComPort.IsOpen Then
            ComPort.Close()
        End If
    End Sub

    Friend Function COM_GetPortNames() As String()
        Return SerialPort.GetPortNames()
    End Function


    ' ==============================================================================
    '   COM SEND
    ' ==============================================================================
    Friend Sub COM_SendText(ByVal text As String)
        If Not ComPort.IsOpen Then Return
        ComPort.WriteLine(text)
    End Sub

    Friend Sub COM_SendString(ByVal text As String, _
                              Optional ByVal terminator As Int32 = 0)
        If Not ComPort.IsOpen Then Return
        If terminator = 1 Then text += vbCr
        If terminator = 2 Then text += vbLf
        If terminator = 3 Then text += vbCrLf
        ComPort.Write(text)
    End Sub

    ' ==============================================================================
    '   COM FUNCTIONS
    ' ==============================================================================
    'Friend Function COM_WaitText(ByVal text As String) As Boolean
    '    If Not ComPort.IsOpen Then Return False
    '    Dim sw As Stopwatch = New Stopwatch
    '    sw.Start()
    '    Do
    '        If COM_ReceivedText.Contains(text) Then
    '            Return True
    '        End If
    '    Loop Until sw.ElapsedMilliseconds > 200   ' timeout in millisecond
    '    Return False
    'End Function

    Friend Function COM_IsOpen() As Boolean
        Return ComPort.IsOpen
    End Function

    Friend Sub COM_DiscardInBuffer()
        If ComPort.IsOpen Then
            ComPort.DiscardInBuffer()
        End If
    End Sub


    ' ==============================================================================
    '   DATA RECEIVED EVENT
    ' ==============================================================================
    Private Sub comPort_DataReceived(ByVal sender As Object, _
                                     ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) _
                                     Handles ComPort.DataReceived
        Try
            If ShowRawChars Then
                While ComPort.BytesToRead > 0
                    Dim c As Char = ChrW(ComPort.ReadChar)
                    DecodeRawChars(c)
                End While
            Else
                Select Case FieldSeparator
                    Case "0", "1"
                        While ComPort.BytesToRead > 0
                            Dim c As Char = ChrW(ComPort.ReadChar)
                            DecodeCaliper01(c)
                        End While
                    Case "2"
                        While ComPort.BytesToRead > 0
                            Dim c As Char = ChrW(ComPort.ReadChar)
                            DecodeCaliper2(c)
                        End While
                    Case Else
                        While ComPort.BytesToRead > 0
                            CharLine = ComPort.ReadLine
                            Decode()
                        End While
                End Select
            End If
        Catch
        End Try
    End Sub

    ' ==============================================================================
    '   DECODE AND SEND TO SLOTS
    ' ==============================================================================
    Friend COM_ReceivedText As String = ""
    Friend FirstSlot As Int32 = 8
    Friend FieldSeparator As String = " "
    Private Fields() As String
    Private CharLine As String
    Private LinesBuffer As String = ""
    Private SleepTime As Single

    ' ==============================================================================
    '   Decode raw chars
    ' ==============================================================================
    Private Sub DecodeRawChars(ByVal c As Char)
        If (c >= Chr(32) And c <= Chr(127)) Then
            CharLine += c
        Else
            CharLine += "<" + AscW(c).ToString + ">"
            If c = vbLf Then
                CharLine += vbLf
            End If
        End If
        ' ---------------------------------------------------------- update LinesBuffer
        UpdateLinesBuffer()
    End Sub


    ' ==============================================================================
    '   Update buffers
    ' ==============================================================================
    Friend ComReceivedTextUpdated As Boolean

    Private LockObj As Object = New Object
    Private Sub UpdateLinesBuffer()
        SyncLock LockObj
            LinesBuffer += CharLine
            CharLine = ""
        End SyncLock
    End Sub

    Friend Sub UpdateComReceivedText()
        ' -------------------------------------------------------------- nothing to do
        If LinesBuffer = "" Then Return
        ' -------------------------------------------------------------- COM_ReceivedText is already updated from calipers and comparators 
        Select Case FieldSeparator
            Case "0", "1", "2"
                ComReceivedTextUpdated = True
                Return
        End Select
        ' -------------------------------------------------------------- update ComReceivedText
        SyncLock LockObj
            COM_ReceivedText = Strings.Right(COM_ReceivedText, 100000) + LinesBuffer
            LinesBuffer = ""
            ComReceivedTextUpdated = True
        End SyncLock
    End Sub


    ' ==============================================================================
    '  Decode --->  " "  ";"  ","  "."
    ' ==============================================================================
    Private Sub Decode() 
        If FirstSlot > 0 Then
            ' ----------------------------------------------------- 
            SyncronizeWriteSlot_WithLineFrequency()
            ' ----------------------------------------------------- separate fields and write slots
            Fields = Split(CharLine, FieldSeparator)
            For i As Int32 = 0 To Fields.Length - 1
                Slots.WriteSlot(FirstSlot + i, CSng(Val(Fields(i).Replace(",", "."))))
            Next
        End If
        ' ---------------------------------------------------------- update LinesBuffer
        UpdateLinesBuffer()
    End Sub

    Private Sub SyncronizeWriteSlot_WithLineFrequency()
        ' ----------------------------------------------------- syncronize the Slots write with input speed
        Dim nbytes As Int32 = ComPort.BytesToRead
        If nbytes < 500 Then
            SleepTime = (400 - nbytes) * 0.05F
            If SleepTime < 0 Then SleepTime = 0
            If SleepTime > 20 Then SleepTime = 20
            Threading.Thread.Sleep(CInt(SleepTime))
        End If
        ' ----------------------------------------------------- send sync values to debug slots
        'Slots.WriteSlot(20, nbytes)
        'Slots.WriteSlot(21, SleepTime)
    End Sub


    ' ==============================================================================
    '   Decode Caliper (0) or Cheap-Comparator (1) data
    ' ==============================================================================
    Private Sub DecodeCaliper01(ByVal c As Char)
        '
        Static OldValue As String = ""
        Static Status As String = "Waiting18"
        Static DigitCounter As Int32
        Static DigitString As String = ""
        '
        Select Case Status
            Case "Waiting18"
                If c = Chr(18) Then Status = "WaitingSign"
            Case "WaitingSign"
                DigitString = c
                Status = "Waiting0"
            Case "Waiting0"
                If c = Chr(0) Then
                    DigitCounter = 1
                    Status = "ReadingDigits"
                Else
                    Status = "Waiting18"
                End If
            Case "ReadingDigits"
                If DigitCounter <= 6 Then
                    DigitString += c
                    DigitCounter += 1
                Else
                    If DigitString <> OldValue Then
                        COM_ReceivedText = Strings.Right(COM_ReceivedText, 100000) + DigitString + vbCrLf
                        OldValue = DigitString
                    End If
                    If FirstSlot > 0 Then
                        If IsNumeric(DigitString) Then
                            Dim n As Single = CSng(Val(DigitString))
                            If n >= -999999 And n <= 999999 Then
                                Slots.WriteSlot(FirstSlot, n / CaliperDivision)
                            End If
                        End If
                    End If
                    Status = "Waiting18"
                End If
        End Select
    End Sub

    ' ==============================================================================
    '   Decode Professional-Comparator(2) data
    ' ==============================================================================
    Private Sub DecodeCaliper2(ByVal c As Char)
        '
        Static DigitString As String = ""
        '
        If c = Chr(13) Then
            DigitString = ""
        Else
            DigitString += c
            If DigitString.Length = 9 Then
                COM_ReceivedText = Strings.Right(COM_ReceivedText, 100000) + DigitString + vbCrLf
                If FirstSlot > 0 Then
                    If IsNumeric(DigitString) Then
                        Dim n As Single = CSng(Val(DigitString))
                        If n >= -999999 And n <= 999999 Then
                            Slots.WriteSlot(FirstSlot, n)
                        End If
                    End If
                End If
            End If
        End If
    End Sub

End Module


